@@ -14,6 +14,7 @@ from eodatasets3.properties import StacPropertyView, EoFields
 
 # TODO: these need discussion.
 DEA_URI_PREFIX = "https://collections.dea.ga.gov.au"
+DEAFRICA_URI_PREFIX = "https://digitalearthafrica.org"
 ODC_DATASET_SCHEMA_URL = "https://schemas.opendatacube.org/dataset"
 
 # Either a local filesystem path or a string URI.
@@ -160,8 +161,6 @@ class ComplicatedNamingConventions:
 
         Example file structure (note version number in file):
             ga_ls8c_ones_3/090/084/2016/01/21/ga_ls8c_ones_3-0-0_090084_2016-01-21_final.odc-metadata.yaml
-
-
         """
         return cls(
             dataset=dataset,
@@ -437,9 +436,15 @@ class ComplicatedNamingConventions:
             # Extract from usgs standard:
             # landsat:landsat_product_id: LC08_L1TP_091075_20161213_20170316_01_T2
             # landsat:landsat_scene_id: LC80910752016348LGN01
-            landsat_id: str = self.dataset.properties.get(
-                "landsat:landsat_product_id"
-            ) or self.dataset.properties.get("landsat:landsat_scene_id")
+            landsat_id = self.dataset.properties.get("landsat:landsat_product_id")
+            if landsat_id is None:
+                landsat_id = self.dataset.properties.get("landsat:landsat_scene_id")
+
+            # from USGS STAC, label is LC08_L2SP_178079_20210417_20210424_02_T1_SR and
+            # landsat:scene_id: LC81780792021107LGN00
+            if landsat_id is None:
+                landsat_id = self.dataset.properties.get("landsat:scene_id")
+
             if not landsat_id:
                 raise NotImplementedError(
                     "TODO: Can only currently abbreviate instruments from Landsat references."
@@ -535,12 +540,28 @@ class ComplicatedNamingConventionsDerivatives(ComplicatedNamingConventions):
                 "odc:producer",
                 "odc:product_family",
                 "odc:region_code",
-                "odc:dataset_version",
                 "dea:dataset_maturity",
             ),
             dataset_separator_field="sentinel:datatake_start_datetime",
         )
 
+    @classmethod
+    def for_deafrica_derivatives(cls, dataset: EoFields, uri=DEAFRICA_URI_PREFIX):
+        """
+        DEAFRICA USGS C2 Naming Convention
+        """
+        return cls(
+            dataset=dataset,
+            base_product_uri=uri,
+            required_fields=(
+                "eo:platform",
+                "odc:producer",
+                "odc:region_code",
+                "odc:product_family",
+                "odc:dataset_version",
+            ),
+        )
+
     def _product_group(self, subname=None) -> str:
         # Computes product group, e.g "ga_ls_wo_3"
         # Deliberately fail if any of these attributes not found.
@@ -549,6 +570,12 @@ class ComplicatedNamingConventionsDerivatives(ComplicatedNamingConventions):
             self.platform_abbreviated,
             self.dataset.product_family,
         ]
+
+        # Exceptional case for DE Africa. There must be a more elegant
+        # way to do this...
+        if self.producer_abbreviated == "deafrica":
+            parts = [self.dataset.product_family, self.platform_abbreviated]
+
         return "_".join(parts)
 
     def destination_folder(self, base: Path) -> Path:
@@ -579,6 +606,7 @@ class ComplicatedNamingConventionsDerivatives(ComplicatedNamingConventions):
             f"{self.dataset.datetime:%Y-%m-%d}",
             self.dataset.maturity,
         ]
+        parts = [x for x in parts if x is not None]
         return "_".join(parts)
 
     @property
