@@ -20,7 +20,8 @@ from eodatasets3 import serialise, utils, DatasetAssembler, IfExists
 from eodatasets3.properties import FileFormat
 from eodatasets3.ui import PathPath
 
-_COPYABLE_MTL_FIELDS = [
+_COPYABLE_MTL_FIELDS = {}
+_COPYABLE_MTL_FIELDS["C1"] = [
     (
         "metadata_file_info",
         (
@@ -47,6 +48,33 @@ _COPYABLE_MTL_FIELDS = [
     ),
     ("projection_parameters", ("scan_gap_interpolation",)),
 ]
+_COPYABLE_MTL_FIELDS["C2"] = [
+    (
+        "level1_processing_record",
+        (
+            "landsat_scene_id",
+            "landsat_product_id",
+            "processing_software_version",
+            "collection_category",
+            "ground_control_points_version",
+            "ground_control_points_model",
+            "geometric_rmse_model_x",
+            "geometric_rmse_model_y",
+        ),
+    ),
+    (
+        "image_attributes",
+        (
+            "station_id",
+            "wrs_path",
+            "wrs_row",
+            # "ground_control_points_verify",  # not in the test data for C1 or C2
+            # "geometric_rmse_verify",   # not in the test data for C1 or C2
+        ),
+    ),
+    # not in the test data for C1 or C2
+    # ("level1_projection_parameters", ("scan_gap_interpolation",)),
+]
 
 # Static namespace to generate uuids for datacube indexing
 USGS_UUID_NAMESPACE = uuid.UUID("276af61d-99f8-4aa3-b2fb-d7df68c5e28f")
@@ -82,6 +110,25 @@ LANDSAT_xTM_BAND_ALIASES = {
 
 MTL_PAIRS_RE = re.compile(r"(\w+)\s=\s(.*)")
 
+LANDSAT_MTL_MAP = {
+    "C1": {
+        "product_contents_cn": "metadata_file_info",
+        "product_contents_of": "product_metadata",
+        "product_contents_fn": "product_metadata",
+        "image_attributes": "product_metadata",
+        "level1_processing_record": "metadata_file_info",
+        "level1_projection_parameters": "projection_parameters",
+    },
+    "C2": {
+        "product_contents_cn": "product_contents",
+        "product_contents_of": "level1_processing_record",
+        "product_contents_fn": "product_contents",
+        "image_attributes": "image_attributes",
+        "level1_processing_record": "level1_processing_record",
+        "level1_projection_parameters": "level1_projection_parameters",
+    },
+}
+
 
 def get_band_alias_mappings(sat: str, instrument: str) -> Dict[str, str]:
     """
@@ -122,9 +169,7 @@ def get_band_alias_mappings(sat: str, instrument: str) -> Dict[str, str]:
     )
 
 
-def get_mtl_content(
-    acquisition_path: Path, root_element="l1_metadata_file"
-) -> Tuple[Dict, str]:
+def get_mtl_content(acquisition_path: Path, root_element=None) -> Tuple[Dict, str, str]:
     """
     Find MTL file for the given path. It could be a directory or a tar file.
 
@@ -151,7 +196,8 @@ def get_mtl_content(
             for member in iter_tar_members(tp):
                 if "_MTL" in member.name:
                     with tp.extractfile(member) as fp:
-                        return read_mtl(fp), member.name
+                        mtl_doc, file_root_element = read_mtl(fp)
+                        return mtl_doc, file_root_element, member.name
             else:
                 raise RuntimeError(
                     "MTL file not found in {}".format(str(acquisition_path))
@@ -167,10 +213,11 @@ def get_mtl_content(
             )
         [path] = paths
         with path.open("r") as fp:
-            return read_mtl(fp, root_element), path.name
+            mtl_doc, file_root_element = read_mtl(fp, root_element)
+            return mtl_doc, file_root_element, path.name
 
 
-def read_mtl(fp: Iterable[Union[str, bytes]], root_element="l1_metadata_file") -> Dict:
+def read_mtl(fp: Iterable[Union[str, bytes]], root_element=None) -> Tuple[Dict, str]:
     def _parse_value(s: str) -> Union[int, float, str]:
         """
         >>> _parse_value("asdf")
@@ -211,12 +258,14 @@ def read_mtl(fp: Iterable[Union[str, bytes]], root_element="l1_metadata_file") -
         return tree
 
     tree = _parse_group(fp)
-    return tree[root_element]
+    if root_element is None:
+        root_element = list(tree.keys())[0]
+    return tree[root_element], root_element
 
 
-def _iter_bands_paths(mtl_doc: Dict) -> Generator[Tuple[str, str], None, None]:
+def _iter_bands_paths(product_doc: Dict) -> Generator[Tuple[str, str], None, None]:
     prefix = "file_name_band_"
-    for name, filepath in mtl_doc["product_metadata"].items():
+    for name, filepath in product_doc.items():
         if not name.startswith(prefix):
             continue
         usgs_band_id = name[len(prefix) :]
@@ -235,23 +284,26 @@ def prepare_and_write(
 
     Input dataset path can be a folder or a tar file.
     """
-    mtl_doc, mtl_filename = get_mtl_content(ds_path)
+    mtl_doc, root_element, mtl_filename = get_mtl_content(ds_path)
     if not mtl_doc:
         raise ValueError(f"No MTL file found for {ds_path}")
-
-    usgs_collection_number = mtl_doc["metadata_file_info"].get("collection_number")
+    collection_key = "C2" if root_element == "landsat_metadata_file" else "C1"
+    coll_map = LANDSAT_MTL_MAP[collection_key]
+    usgs_collection_number = mtl_doc[coll_map["product_contents_cn"]].get(
+        "collection_number"
+    )
     if usgs_collection_number is None:
         raise NotImplementedError(
             "Dataset has no collection number: pre-collection data is not supported."
         )
 
-    data_format = mtl_doc["product_metadata"]["output_format"]
+    data_format = mtl_doc[coll_map["product_contents_of"]]["output_format"]
     if data_format.upper() != "GEOTIFF":
         raise NotImplementedError(f"Only GTiff currently supported, got {data_format}")
     file_format = FileFormat.GeoTIFF
 
     # Assumed below.
-    projection_params = mtl_doc["projection_parameters"]
+    projection_params = mtl_doc[coll_map["level1_projection_parameters"]]
     if (
         "grid_cell_size_thermal" in projection_params
         and "grid_cell_size_reflective" in projection_params
@@ -272,7 +324,8 @@ def prepare_and_write(
         dataset_location=ds_path,
         # Detministic ID based on USGS's product id (which changes when the scene is reprocessed by them)
         dataset_id=uuid.uuid5(
-            USGS_UUID_NAMESPACE, mtl_doc["metadata_file_info"]["landsat_product_id"]
+            USGS_UUID_NAMESPACE,
+            mtl_doc[coll_map["level1_processing_record"]]["landsat_product_id"],
         ),
         naming_conventions="dea",
         if_exists=IfExists.Overwrite,
@@ -282,15 +335,21 @@ def prepare_and_write(
             assert producer == "ga.gov.au"
             p.add_source_path(source_telemetry)
 
-        p.platform = mtl_doc["product_metadata"]["spacecraft_id"]
-        p.instrument = mtl_doc["product_metadata"]["sensor_id"]
+        p.platform = mtl_doc[coll_map["image_attributes"]]["spacecraft_id"]
+        p.instrument = mtl_doc[coll_map["image_attributes"]]["sensor_id"]
         p.product_family = "level1"
         p.producer = producer
         p.datetime = "{}T{}".format(
-            mtl_doc["product_metadata"]["date_acquired"],
-            mtl_doc["product_metadata"]["scene_center_time"],
+            mtl_doc[coll_map["image_attributes"]]["date_acquired"],
+            mtl_doc[coll_map["image_attributes"]]["scene_center_time"],
         )
-        p.processed = mtl_doc["metadata_file_info"]["file_date"]
+        if collection_key == "C2":
+            p.processed = mtl_doc["level1_processing_record"]["date_product_generated"]
+            p.properties["landsat:data_type"] = mtl_doc["level1_processing_record"][
+                "processing_level"
+            ]
+        else:
+            p.processed = mtl_doc["metadata_file_info"]["file_date"]
         p.properties["odc:file_format"] = file_format
         p.properties["eo:gsd"] = ground_sample_distance
         cloud_cover = mtl_doc["image_attributes"]["cloud_cover"]
@@ -300,7 +359,7 @@ def prepare_and_write(
         p.properties["eo:sun_azimuth"] = mtl_doc["image_attributes"]["sun_azimuth"]
         p.properties["eo:sun_elevation"] = mtl_doc["image_attributes"]["sun_elevation"]
         p.properties["landsat:collection_number"] = usgs_collection_number
-        for section, fields in _COPYABLE_MTL_FIELDS:
+        for section, fields in _COPYABLE_MTL_FIELDS[collection_key]:
             for field in fields:
                 value = mtl_doc[section].get(field)
                 if value is not None:
@@ -308,24 +367,32 @@ def prepare_and_write(
 
         p.region_code = f"{p.properties['landsat:wrs_path']:03d}{p.properties['landsat:wrs_row']:03d}"
         org_collection_number = utils.get_collection_number(
-            p.producer, p.properties["landsat:collection_number"]
+            p.platform, p.producer, p.properties["landsat:collection_number"]
         )
         p.dataset_version = f"{org_collection_number}.0.{p.processed:%Y%m%d}"
 
         # NRT product?
         # Category is one of: T1, T2 or RT ('real time')
         if p.properties["landsat:collection_category"] == "RT":
-            p.properties["odc:dataset_maturity"] = "nrt"
+            p.properties["dea:dataset_maturity"] = "nrt"
 
         band_aliases = get_band_alias_mappings(p.platform, p.instrument)
-        for usgs_band_id, file_location in _iter_bands_paths(mtl_doc):
+        for usgs_band_id, file_location in _iter_bands_paths(
+            mtl_doc[coll_map["product_contents_fn"]]
+        ):
             p.note_measurement(
                 band_aliases[usgs_band_id],
                 file_location,
                 relative_to_dataset_location=True,
                 expand_valid_data=usgs_band_id != "quality",
             )
-
+        if collection_key == "C2":
+            p.note_measurement(
+                band_aliases["quality"],
+                mtl_doc[coll_map["product_contents_fn"]]["file_name_quality_l1_pixel"],
+                relative_to_dataset_location=True,
+                expand_valid_data=False,
+            )
         p.add_accessory_file("metadata:landsat_mtl", Path(mtl_filename))
 
         return p.done()
@@ -451,7 +518,8 @@ def _normalise_dataset_path(input_path: Path) -> Path:
             return input_path
         input_path = input_path.parent
 
-    mtl_files = list(input_path.rglob("*_MTL*"))
+    mtl_files = list(input_path.rglob("*_MTL.txt"))
+
     if not mtl_files:
         raise ValueError(
             "No MTL files within input path '{}'. Not a dataset?".format(input_path)
